/*-
 * Copyright (C) 2007 Erik Larsson
 * 
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package org.catacombae.findduplicates;

import org.catacombae.findduplicates.gui.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import net.iharder.dnd.FileDrop;

public class MainWindow extends JFrame {
    private SelectSourcesPanel ssPanel;
    private ViewResultsPanel vrPanel;
    private ProgressDialog pDialog;
    private CardLayout layout;
    private JFileChooser jfc;
    
    private LinkedList<File> sourceList = new LinkedList<File>();

    public MainWindow() {
	super("FindDuplicates " + FindDuplicatesGUI.VERSION);
	ssPanel = new SelectSourcesPanel();
	vrPanel = new ViewResultsPanel();
	pDialog = new ProgressDialog(this);
	layout = new CardLayout();

	ssPanel.addButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    actionAddSource();
		}
	    });
	ssPanel.removeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    actionRemoveSource();
		}
	    });
	ssPanel.clearButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    actionClearSources();
		}
	    });
	ssPanel.findButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    actionFindDuplicates();
		}
	    });

	vrPanel.backButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    layout.show(getContentPane(), "A");
		}
	    });
	vrPanel.exitButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    System.exit(0);
		}
	    });
	
	new FileDrop(ssPanel, new FileDrop.Listener() {
		public void filesDropped(java.io.File[] files) {   
		    // handle file drop
		    if(files.length > 0)
			addFilesToList(files);
		}   // end filesDropped
	    }); // end FileDrop.Listener
	

	getContentPane().setLayout(layout);
	getContentPane().add(ssPanel, "A");
	getContentPane().add(vrPanel, "B");
	layout.show(getContentPane(), "A");

	pack();
	setLocationRelativeTo(null);
	setDefaultCloseOperation(EXIT_ON_CLOSE);
    }
    
    public void actionAddSource() {
	if(jfc == null) {
	    jfc = new JFileChooser();
	    jfc.setMultiSelectionEnabled(true);
	    jfc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
	}
	if(jfc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
	    File[] selectedFiles = jfc.getSelectedFiles();
	    addFilesToList(selectedFiles);
	}
    }

    private void addFilesToList(File[] files) {
	for(File f : files)
	    sourceList.add(f);
	refreshSourceList();
    }

    public void actionRemoveSource() {
	LinkedList<File> copy = new LinkedList<File>();
	copy.addAll(sourceList);
	sourceList.clear();
	int[] indices = ssPanel.sourceList.getSelectedIndices();
	int i = 0;
	for(File f : copy) {
	    boolean removed = false;
	    for(int j = 0; j < indices.length; ++j)
		if(indices[j] == i) removed = true;
	    if(!removed)
		sourceList.add(f);
	    ++i;
	}
	refreshSourceList();
    }
    
    public void actionClearSources() {
	sourceList.clear();
	refreshSourceList();
    }
    
    public void actionFindDuplicates() {
	if(sourceList.isEmpty()) {
	    JOptionPane.showMessageDialog(this, "No sources in list.", "Error", JOptionPane.ERROR_MESSAGE);
	    return;
	}
	
	Runnable r = new Runnable() {
		public void run() {
		    File[] allFiles = FindDuplicatesEngine.resolveDirectories(sourceList.toArray(new File[sourceList.size()]), pDialog);
		    if(allFiles != null) {
			try {
			    FindDuplicatesEngine.Result[] results = FindDuplicatesEngine.findDuplicates(allFiles, ssPanel.hashAlgorithmBox.getSelectedItem().toString(), pDialog);
			    pDialog.setVisible(false);
			    if(results != null) {
				layout.show(getContentPane(), "B");
				vrPanel.resultArea.setText("");
				vrPanel.resultArea.append("Report:\n");
				if(results.length == 0)
				    vrPanel.resultArea.append("No duplicates found.\n");
				else {
				    int i = 1;
				    for(FindDuplicatesEngine.Result r : results) {
					vrPanel.resultArea.append("[" + i++ + "] The following files are identical:\n");
					for(FindDuplicatesEngine.Entry e : r.matchingEntries) {
					    vrPanel.resultArea.append("    " + e.file.getAbsolutePath() + "\n");
					    
					}
				    }
				}
			    }
			} catch(Exception e) {
			    JOptionPane.showMessageDialog(MainWindow.this.getContentPane(), e.toString(), "Exception", JOptionPane.ERROR_MESSAGE);
			    e.printStackTrace();
			    pDialog.setVisible(false);
			}
		    }
		    else
			pDialog.setVisible(false);
		}
	    };
	new Thread(r).start();
	pDialog.reset();
	pDialog.setVisible(true); // THIS SUCKS, FIX IT...
    }
    
    public void refreshSourceList() {
	//System.out.println("refreshSourceList()");
	String[] stringList = new String[sourceList.size()];
	int i = 0;
	for(File f : sourceList) {
	    //System.out.println(" " + f);
	    String current = f.getAbsolutePath();
	    if(f.isDirectory())
		stringList[i++] = current + " [dir]";
	    else
		stringList[i++] = current;
	}
	ssPanel.sourceList.setListData(stringList);
// 	ssPanel.sourceList.
// 	for(File f : sourceList)
    }
}
